package com.iconplus.smartproc.service.approval;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.DrpApproval;
import com.iconplus.smartproc.model.projection.DrpApprovalView;
import com.iconplus.smartproc.model.request.DrpApprovalRequest;
import com.iconplus.smartproc.model.response.DrpApprovalResponse;
import com.iconplus.smartproc.model.response.GetListDrpApprovalResponse;
import com.iconplus.smartproc.repository.DrpApprovalRepository;
import com.iconplus.smartproc.repository.DrpRekomendasiRepository;
import com.iconplus.smartproc.repository.DrpRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.sql.Date;
import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class GetListApprovalDrpServie implements BaseService<DrpApprovalRequest, GetListDrpApprovalResponse> {

    private final DrpRepository drpRepository;
    private final DrpApprovalRepository drpApprovalRepository;

    public GetListApprovalDrpServie(DrpRepository drpRepository,
                                    DrpApprovalRepository drpApprovalRepository) {
        this.drpApprovalRepository = drpApprovalRepository;
        this.drpRepository = drpRepository;

    }

    @Override
    public GetListDrpApprovalResponse execute(DrpApprovalRequest input) {

        List<DrpApprovalResponse> drpApprovalResponseList = new ArrayList<>();
        var drpApprovalViews = drpRepository.getListApprovalDrp(input.getPageable());
        for (DrpApprovalView drpApprovalView : drpApprovalViews) {
            DrpApprovalResponse drpApprovalResponse = DrpApprovalResponse.builder()
                    .id(drpApprovalView.getId())
                    .drpId(drpApprovalView.getDrpId())
                    .tahun(drpApprovalView.getTahun())
                    .approveStatus(drpApprovalView.getApproveStatus())
                    .approverVpDate(drpApprovalView.getVpApproveDate())
                    .approverDirekturDate(drpApprovalView.getDirekturApproveDate())
                    .approverKomiteDate(drpApprovalView.getKomiteApproveDate())
                    .build();
            drpApprovalResponseList.add(drpApprovalResponse);
        }

        return GetListDrpApprovalResponse.builder()
                .data(drpApprovalResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(drpApprovalViews.getTotalPages())
                        .totalRecords(drpApprovalViews.getTotalElements())
                        .isFirstPage(drpApprovalViews.isFirst())
                        .isLastPage(drpApprovalViews.isLast())
                        .build())
                .build();
    }
}
