package com.iconplus.smartproc.service.drp;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.model.EmptyResponse;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.DrpDokumenRequest;
import com.iconplus.smartproc.model.request.DrpRequest;
import com.iconplus.smartproc.model.response.DrpDokumenResponse;
import com.iconplus.smartproc.repository.DrpDokumenRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

@Service
@Slf4j
public class EditUploadDokumenDrpService implements BaseService<DrpDokumenRequest, DrpDokumenResponse> {

    @Value("${config.file-upload.rkap}")
    private String pathFileUploadRkap;

    private final DrpDokumenRepository drpDokumenRepository;

    public EditUploadDokumenDrpService(DrpDokumenRepository drpDokumenRepository) {
        this.drpDokumenRepository = drpDokumenRepository;
    }

    @Override
    public DrpDokumenResponse execute(DrpDokumenRequest input) {

        MultipartFile file = input.getFile();
        String newFilename = file.getOriginalFilename();

        var drpDokumenOptional = drpDokumenRepository.findByFilenameAndIsDeleteFalse(input.getFilename());
        if (drpDokumenOptional.isPresent()) {
            drpDokumenOptional.get().setDrpId(input.getDrpId());
            drpDokumenOptional.get().setJenisDokumenId(input.getJenisDokumenId());
            drpDokumenOptional.get().setFilename(newFilename);
            drpDokumenOptional.get().setKeterangan(input.getKeterangan());
            drpDokumenRepository.save(drpDokumenOptional.get());
        }

        Path oldFile = Paths.get(pathFileUploadRkap + input.getFilename());
        try {
            Files.delete(oldFile);
        } catch (IOException e) {
            log.info("no file found");
        }

        File path = new File(pathFileUploadRkap + newFilename);

        try {
            FileOutputStream output = new FileOutputStream(path);
            output.write(file.getBytes());
            output.close();
        } catch (Exception ex) {
            throw new BusinessException(Constants.ERR_CODE_10012,
                    Constants.ERR_TTL_10012,
                    String.format(Constants.ERR_MSG_10012, pathFileUploadRkap));
        }

        return DrpDokumenResponse.builder()
                .jenisDokumenId(input.getJenisDokumenId())
                .filename(newFilename)
                .keterangan(input.getKeterangan())
                .build();
    }
}
