package com.iconplus.smartproc.service.drppengadaan;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.request.DrpPengadaanDokumenRequest;
import com.iconplus.smartproc.model.response.DrpPengadaanDokumenResponse;
import com.iconplus.smartproc.repository.DrpPengadaanDokumenRepository;
import com.iconplus.smartproc.util.Constants;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

@Service
@Slf4j
public class EditUploadDrpPengadaanService implements BaseService<DrpPengadaanDokumenRequest, DrpPengadaanDokumenResponse> {

    @Value("${config.file-upload.pendukung}")
    private String pathFileUploadPendukung;

    private final DrpPengadaanDokumenRepository drpPengadaanDokumenRepository;
    public EditUploadDrpPengadaanService(DrpPengadaanDokumenRepository drpPengadaanDokumenRepository) {
        this.drpPengadaanDokumenRepository = drpPengadaanDokumenRepository;
    }

    @Override
    public DrpPengadaanDokumenResponse execute(DrpPengadaanDokumenRequest input) {
        MultipartFile file = input.getFile();
        String newFilename = file.getOriginalFilename();

        var drpDokumenOptional = drpPengadaanDokumenRepository.findByFilenameAndIsDeleteFalse(input.getFilename());
        if (drpDokumenOptional.isPresent()) {
            drpDokumenOptional.get().setDrpId(input.getDrpId());
            drpDokumenOptional.get().setJenisDokumenId(input.getJenisDokumenId());
            drpDokumenOptional.get().setFilename(newFilename);
            drpDokumenOptional.get().setKeterangan(input.getKeterangan());
            drpPengadaanDokumenRepository.save(drpDokumenOptional.get());
        }

        Path oldFile = Paths.get(pathFileUploadPendukung + input.getFilename());
        try {
            Files.delete(oldFile);
        } catch (IOException e) {
            log.info("no file found");
        }

        File path = new File(pathFileUploadPendukung + newFilename);

        try {
            FileOutputStream output = new FileOutputStream(path);
            output.write(file.getBytes());
            output.close();
        } catch (Exception ex) {
            throw new BusinessException(Constants.ERR_CODE_10012,
                    Constants.ERR_TTL_10012,
                    String.format(Constants.ERR_MSG_10012, pathFileUploadPendukung));
        }

        return DrpPengadaanDokumenResponse.builder()
                .jenisDokumenId(input.getJenisDokumenId())
                .filename(newFilename)
                .keterangan(input.getKeterangan())
                .build();
    }
}
