package com.iconplus.smartproc.service.rks;

import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.entity.RksIsi;
import com.iconplus.smartproc.model.request.RksIsiRequest;
import com.iconplus.smartproc.model.response.ListRksIsiResponse;
import com.iconplus.smartproc.model.response.RksIsiResponse;
import com.iconplus.smartproc.repository.RksIsiRepository;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListRksIsiService implements BaseService<RksIsiRequest, ListRksIsiResponse> {

    private final RksIsiRepository rksIsiRepository;

    public GetListRksIsiService(RksIsiRepository rksIsiRepository) {
        this.rksIsiRepository = rksIsiRepository;
    }

    @Override
    public ListRksIsiResponse execute(RksIsiRequest input) {

        List<RksIsiResponse> rksIsiResponseList = new ArrayList<>();
        var rksIsis = rksIsiRepository.findByRksDaftarIsiIdAndIsDeleteFalse(input.getRksDaftarIsiId(), input.getPageable());
        for (RksIsi rksIsi : rksIsis) {
            RksIsiResponse rksIsiResponse = RksIsiResponse.builder()
                    .id(rksIsi.getId())
                    .rksDaftarIsiId(rksIsi.getRksDaftarIsiId())
                    .nomor(rksIsi.getNomor())
                    .parentId(rksIsi.getParentId())
                    .nama(rksIsi.getNama())
                    .bab(rksIsi.getBab())
                    .subBab(rksIsi.getSubBab())
                    .urutan(rksIsi.getUrutan())
                    .isKontrak(rksIsi.getIsKontrak())
                    .build();
            rksIsiResponseList.add(rksIsiResponse);
        }

        return ListRksIsiResponse.builder()
                .data(rksIsiResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(rksIsis.getTotalPages())
                        .totalRecords(rksIsis.getTotalElements())
                        .isFirstPage(rksIsis.isFirst())
                        .isLastPage(rksIsis.isLast())
                        .build())
                .build();
    }
}
