package com.iconplus.smartproc.service.rks;

import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.projection.RksTemplateView;
import com.iconplus.smartproc.model.request.RksTemplateRequest;
import com.iconplus.smartproc.model.response.ListRksTemplateResponse;
import com.iconplus.smartproc.model.response.RksTemplateResponse;
import com.iconplus.smartproc.repository.RksTemplateRepository;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListRksTemplateService implements BaseService<RksTemplateRequest, ListRksTemplateResponse> {

    private final RksTemplateRepository rksTemplateRepository;
    public GetListRksTemplateService(RksTemplateRepository rksTemplateRepository) {
        this.rksTemplateRepository = rksTemplateRepository;
    }

    @Override
    public ListRksTemplateResponse execute(RksTemplateRequest input) {

        var rksTemplateViews = rksTemplateRepository.getListRksTemplate(input.getPageable());

        List<RksTemplateResponse> rksTemplateResponseList = new ArrayList<>();
        for (RksTemplateView rksTemplateView : rksTemplateViews) {
            RksTemplateResponse rksTemplateResponse = RksTemplateResponse.builder()
                    .id(rksTemplateView.getId())
                    .kodeTemplate(rksTemplateView.getKodeTemplate())
                    .namaTemplate(rksTemplateView.getNamaTemplate())
                    .metodePengadaanId(rksTemplateView.getMetodePengadaanId())
                    .metodePengadaan(rksTemplateView.getMetodePengadaan())
                    .versi(rksTemplateView.getVersi())
                    .rksDaftarIsiId(rksTemplateView.getRksDaftarIsiId())
                    .daftarIsi(rksTemplateView.getDaftarIsi())
                    .isActive(rksTemplateView.getIsActive())
                    .build();
            rksTemplateResponseList.add(rksTemplateResponse);
        }

        return ListRksTemplateResponse.builder()
                .data(rksTemplateResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(rksTemplateViews.getTotalPages())
                        .totalRecords(rksTemplateViews.getTotalElements())
                        .isFirstPage(rksTemplateViews.isFirst())
                        .isLastPage(rksTemplateViews.isLast())
                        .build())
                .build();
    }
}
